/*
BMP180 lib 0x01

copyright (c) Davide Gironi, 2012

Released under GPLv3.
Please refer to LICENSE file for licensing information.
*/


#ifndef F_CPU
#define F_CPU	16000000
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <avr/io.h>
#include <util/delay.h>

#include "bmp180.h"

//path to i2c fleury lib
#include BMP180_I2CFLEURYPATH


#if BMP180_FILTERPRESSURE == 1
#define BMP180_AVARAGECOEF 21
static long k[BMP180_AVARAGECOEF];
long BMP180_avaragefilter(long input) {
	uint8_t i=0;
	long sum=0;
	for (i=0; i<BMP180_AVARAGECOEF; i++) {
		k[i] = k[i+1];
	}
	k[BMP180_AVARAGECOEF-1] = input;
	for (i=0; i<BMP180_AVARAGECOEF; i++) {
		sum += k[i];
	}
	return (sum /BMP180_AVARAGECOEF) ;
}
#endif

/*
 * read calibration registers
 */
void BMP180_getcalibration() {
	uint8_t buff[2], ii;
	memset(buff, 0, sizeof(buff));

	I2C_read_multiple(buff, BMP180_ADDR, BMP180_REGAC1, 2, ii);
	BMP180_regac1 = ((int)buff[0] <<8 | ((int)buff[1]));
	
	I2C_read_multiple(buff, BMP180_ADDR, BMP180_REGAC2, 2, ii);
	BMP180_regac2 = ((int)buff[0] <<8 | ((int)buff[1]));
	
	I2C_read_multiple(buff, BMP180_ADDR, BMP180_REGAC3, 2, ii);
	BMP180_regac3 = ((int)buff[0] <<8 | ((int)buff[1]));
	
	I2C_read_multiple(buff, BMP180_ADDR, BMP180_REGAC4, 2, ii);
	BMP180_regac4 = ((unsigned int)buff[0] <<8 | ((unsigned int)buff[1]));
	
	I2C_read_multiple(buff, BMP180_ADDR, BMP180_REGAC5, 2, ii);
	BMP180_regac5 = ((unsigned int)buff[0] <<8 | ((unsigned int)buff[1]));
	
	I2C_read_multiple(buff, BMP180_ADDR, BMP180_REGAC6, 2, ii);
	BMP180_regac6 = ((unsigned int)buff[0] <<8 | ((unsigned int)buff[1]));
	
	I2C_read_multiple(buff, BMP180_ADDR, BMP180_REGB1, 2, ii);
	BMP180_regb1 = ((int)buff[0] <<8 | ((int)buff[1]));
	
	I2C_read_multiple(buff, BMP180_ADDR, BMP180_REGB2, 2, ii);
	BMP180_regb2 = ((int)buff[0] <<8 | ((int)buff[1]));
	
	I2C_read_multiple(buff, BMP180_ADDR, BMP180_REGMB, 2, ii);
	BMP180_regmb = ((int)buff[0] <<8 | ((int)buff[1]));
	
	I2C_read_multiple(buff, BMP180_ADDR, BMP180_REGMC, 2, ii);
	BMP180_regmc = ((int)buff[0] <<8 | ((int)buff[1]));
	
	I2C_read_multiple(buff, BMP180_ADDR, BMP180_REGMD, 2, ii);
	BMP180_regmd = ((int)buff[0] <<8 | ((int)buff[1]));
}

/*
 * get raw temperature as read by registers, and do some calculation to convert it
 */
void BMP180_getrawtemperature() {
	uint8_t buff[2], ii;
	memset(buff, 0, sizeof(buff));
	long ut,x1,x2;

	//read raw temperature
	I2C_write(BMP180_ADDR, BMP180_REGCONTROL, BMP180_REGREADTEMPERATURE);
	_delay_ms(5); // min. 4.5ms read Temp delay
	I2C_read_multiple(buff, BMP180_ADDR, BMP180_REGCONTROLOUTPUT, 2, ii);
	ut = ((long)buff[0] << 8 | ((long)buff[1])); //uncompensated temperature value

	//calculate raw temperature
	x1 = ((long)ut - BMP180_regac6) * BMP180_regac5 >> 15;
	x2 = ((long)BMP180_regmc << 11) / (x1 + BMP180_regmd);
	BMP180_rawtemperature = x1 + x2;
}

/*
 * get raw pressure as read by registers, and do some calculation to convert it
 */
void BMP180_getrawpressure() {
	uint8_t buff[3], ii;
	memset(buff, 0, sizeof(buff));
	long up,x1,x2,x3,b3,b6,p;
	unsigned long b4,b7;

	#if BMP180_AUTOUPDATETEMP == 1
	BMP180_getrawtemperature();
	#endif

	//read raw pressure
	I2C_write(BMP180_ADDR, BMP180_REGCONTROL, BMP180_REGREADPRESSURE+(BMP180_MODE << 6));
	_delay_ms(2 + (3<<BMP180_MODE));
	I2C_read_multiple(buff, BMP180_ADDR, BMP180_REGCONTROLOUTPUT, 3, ii);
	up = ((((long)buff[0] <<16) | ((long)buff[1] <<8) | ((long)buff[2])) >> (8-BMP180_MODE)); // uncompensated pressure value

	//calculate raw pressure
	b6 = BMP180_rawtemperature - 4000;
	x1 = (BMP180_regb2* (b6 * b6) >> 12) >> 11;
	x2 = (BMP180_regac2 * b6) >> 11;
	x3 = x1 + x2;
	b3 = (((((long)BMP180_regac1) * 4 + x3) << BMP180_MODE) + 2) >> 2;
	x1 = (BMP180_regac3 * b6) >> 13;
	x2 = (BMP180_regb1 * ((b6 * b6) >> 12)) >> 16;
	x3 = ((x1 + x2) + 2) >> 2;
	b4 = (BMP180_regac4 * (uint32_t)(x3 + 32768)) >> 15;
	b7 = ((uint32_t)up - b3) * (50000 >> BMP180_MODE);
	p = b7 < 0x80000000 ? (b7 << 1) / b4 : (b7 / b4) << 1;
	x1 = (p >> 8) * (p >> 8);
	x1 = (x1 * 3038) >> 16;
	x2 = (-7357 * p) >> 16;
	BMP180_rawpressure = p + ((x1 + x2 + 3791) >> 4);

	#if BMP180_FILTERPRESSURE == 1
	BMP180_rawpressure = BMP180_avaragefilter(BMP180_rawpressure);
	#endif
}

/*
 * get celsius temperature
 */
double BMP180_gettemperature() {
	BMP180_getrawtemperature();
	double temperature = ((BMP180_rawtemperature + 8)>>4);
	temperature = temperature /10;
	return temperature;
}

/*
 * get pressure
 */
int32_t BMP180_getpressure() {
	BMP180_getrawpressure();
	return BMP180_rawpressure + BMP180_UNITPAOFFSET;
}

/*
 * get altitude
 */
double BMP180_getaltitude() {
	BMP180_getrawpressure();
	return ((1 - pow(BMP180_rawpressure/(double)101325, 0.1903 )) / 0.0000225577) + BMP180_UNITMOFFSET;
}

/*
 * init BMP180
 */
void BMP180_init() {
	#if BMP180_I2CINIT == 1
	//init i2c
	I2C_init();
	_delay_us(10);
	#endif

	BMP180_getcalibration(); //get calibration data
	BMP180_getrawtemperature(); //update raw temperature, at least the first time

	#if BMP180_FILTERPRESSURE == 1
	//initialize the avarage filter
	uint8_t i=0;
	for (i=0; i<BMP180_AVARAGECOEF; i++) {
		BMP180_getrawpressure();
	}
	#endif
}
