/*
BMP180 lib 0x01

copyright (c) Davide Gironi, 2012

Released under GPLv3.
Please refer to LICENSE file for licensing information.

References:
  - this library is a porting of the BMP180driver 0.4 ardunio library
    http://code.google.com/p/BMP180driver/

Notes:
  - to compile with avrgcc you may define -lm flag for some math functions
*/


#ifndef BMP180_H_
#define BMP180_H_

#include <stdio.h>
#include <avr/io.h>
#define BMP180_ADDR (0x77) //0x77 default I2C address


#define BMP180_I2CFLEURYPATH "i2cmaster.h" //define the path to i2c fleury lib
#define BMP180_I2CINIT 1 //init i2c

//registers
#define BMP180_REGAC1 0xAA
#define BMP180_REGAC2 0xAC
#define BMP180_REGAC3 0xAE
#define BMP180_REGAC4 0xB0
#define BMP180_REGAC5 0xB2
#define BMP180_REGAC6 0xB4
#define BMP180_REGB1 0xB6
#define BMP180_REGB2 0xB8
#define BMP180_REGMB 0xBA
#define BMP180_REGMC 0xBC
#define BMP180_REGMD 0xBE
#define BMP180_REGCONTROL 0xF4 //control
#define BMP180_REGCONTROLOUTPUT 0xF6 //output 0xF6=MSB, 0xF7=LSB, 0xF8=XLSB
#define BMP180_REGREADTEMPERATURE 0x2E //read temperature
#define BMP180_REGREADPRESSURE 0x34 //read pressure

//modes
#define BMP180_MODEULTRALOWPOWER 0 //oversampling=0, internalsamples=1, maxconvtimepressure=4.5ms, avgcurrent=3uA, RMSnoise_hPA=0.06, RMSnoise_m=0.5
#define BMP180_MODESTANDARD 1 //oversampling=1, internalsamples=2, maxconvtimepressure=7.5ms, avgcurrent=5uA, RMSnoise_hPA=0.05, RMSnoise_m=0.4
#define BMP180_MODEHIGHRES 2 //oversampling=2, internalsamples=4, maxconvtimepressure=13.5ms, avgcurrent=7uA, RMSnoise_hPA=0.04, RMSnoise_m=0.3
#define BMP180_MODEULTRAHIGHRES 3 //oversampling=3, internalsamples=8, maxconvtimepressure=25.5ms, avgcurrent=12uA, RMSnoise_hPA=0.03, RMSnoise_m=0.25

//autoupdate temperature enabled
#define BMP180_AUTOUPDATETEMP 0 //autoupdate temperature every read

//setup parameters
#define BMP180_MODE BMP180_MODEULTRALOWPOWER //define a mode
#define BMP180_UNITPAOFFSET 0 //define a unit offset (pa)
#define BMP180_UNITMOFFSET 0 //define a unit offset (m)

//avarage filter
#define BMP180_FILTERPRESSURE 1 //avarage filter for pressure

//variables
int BMP180_regac1, BMP180_regac2, BMP180_regac3, BMP180_regb1, BMP180_regb2, BMP180_regmb, BMP180_regmc, BMP180_regmd;
unsigned int BMP180_regac4, BMP180_regac5, BMP180_regac6;
long BMP180_rawtemperature, BMP180_rawpressure;

//functions
extern void BMP180_init();
extern int32_t BMP180_getpressure();
extern double BMP180_getaltitude();
extern double BMP180_gettemperature();

#endif
